;; NAME
;;      hexGIMP
;;
;; VERSION
;;      0.0.1, 2009/04/13
;;
;; DESCRIPTION
;;	hexGIMP is a Script-Fu plugin for GIMP.  It creates a set of layers
;;      and an image grid suitable for making hex maps with brushes derived
;;      from AKS Hex Mapper tiles.
;;
;;      Parameters:
;;          hex rows
;;          hex columns
;;
;; AUTHOR
;;      user "isomage" of the cartographersguild.com forums
;;
;; LICENSE
;;      hexGIMP is free software.

(define hex_width  33)
(define hex_height 27)
(define hex_1_x    25)
(define hex_2_x     7)
(define hex_0_y    13)

(define (hex-origin i j)
  (list (* i (+ hex_1_x 1))
	(if (zero? (modulo i 2))
	    (* j (+ 1 hex_height))
	    (+ (* j (+ 1 hex_height)) hex_0_y 1))))

(define (hex-center i j)
  (let ((origin (hex-origin i j)))
    (list (+ (car  origin) (/ hex_width  2))
	  (+ (cadr origin) (/ hex_height 2)))))

(define (fill-hex-grid drawable cols rows hex)
  (gimp-brushes-set-brush hex)
  (let ((y 0))
    (while (< y rows)
	   (let ((x 0))
	     (while (< x cols)
		    (let ((center (hex-center x y))
			  (coords (cons-array 2 'double)))
		      (aset coords 0 (car  center))
		      (aset coords 1 (cadr center))
		      (gimp-pencil drawable 2 coords))
		    (set! x (+ 1 x))))
	   (set! y (+ 1 y)))))

(define (new-hex-map cols rows)
  (let ((width  (+ (* (+ hex_1_x 1) cols) hex_2_x))
	(height (+ (* (+ hex_height 1) rows) (+ hex_0_y 1))))
    (let* ((image        (car (gimp-image-new width height RGB)))
	   (layer-terrain (car (gimp-layer-new image width height RGBA-IMAGE "Terrain" 100 NORMAL-MODE)))
	   (layer-rivers  (car (gimp-layer-new image width height RGBA-IMAGE "Rivers"  100 NORMAL-MODE)))
	   (layer-roads   (car (gimp-layer-new image width height RGBA-IMAGE "Roads"   100 NORMAL-MODE)))
	   (layer-cities  (car (gimp-layer-new image width height RGBA-IMAGE "Cities"  100 NORMAL-MODE)))
	   (layer-grid    (car (gimp-layer-new image width height RGBA-IMAGE "Grid"    100 NORMAL-MODE)))
	   (layer-borders (car (gimp-layer-new image width height RGBA-IMAGE "Borders" 100 NORMAL-MODE))))
 
      (gimp-image-add-layer image layer-terrain -1)
      (gimp-image-add-layer image layer-rivers  -1)
      (gimp-image-add-layer image layer-roads   -1)
      (gimp-image-add-layer image layer-cities  -1)
      (gimp-image-add-layer image layer-grid    -1)
      (gimp-image-add-layer image layer-borders -1)

      (gimp-edit-clear layer-terrain)
      (gimp-edit-clear layer-rivers)
      (gimp-edit-clear layer-roads)
      (gimp-edit-clear layer-cities)
      (gimp-edit-clear layer-grid)
      (gimp-edit-clear layer-borders)

      (fill-hex-grid layer-terrain rows cols "hex clear")

      (let ((foreground (car (gimp-context-get-foreground))))
	(gimp-context-set-foreground '(128 128 128))
	(gimp-by-color-select layer-terrain '(255 255 255) 0 0 FALSE FALSE 0 FALSE)
	(gimp-selection-invert image)
	(gimp-bucket-fill layer-grid FG-BUCKET-FILL NORMAL-MODE 100 0 FALSE 0 0)
	(gimp-selection-none image)
	(gimp-context-set-foreground foreground))

      (gimp-image-grid-set-spacing image 26 14)
      (gimp-image-grid-set-offset  image 16 -1)
      (gimp-image-grid-set-style   image GRID-DOTS)

      (gimp-image-set-active-layer image layer-terrain)

      (gimp-display-new image)
)))

(script-fu-register "new-hex-map"
		    "<Toolbox>/File/New Hex Map"
		    "Create a new hex map"
		    "isomage"
		    "isomage"
		    "Created 2009/04/13, version 0.0.1 2009/04/13"
		    ""

		    SF-VALUE "Columns" "16"
		    SF-VALUE "Rows"    "16"
)
